@@ -1,9 +1,16 @@
-use std::{fs::File, io::Write, path::Path};
-
 use mytest::*;
 use rstest_test::*;
+use std::{fs::File, io::Write, path::Path};
 use unindent::Unindent;
 
+const SRC_LIB_RS: &str = {
+    const SEP: u8 = std::path::MAIN_SEPARATOR as u8;
+    const SRC_LIB_RS: [u8; 11] = [
+        SEP, b's', b'r', b'c', SEP, b'l', b'i', b'b', b'.', b'r', b's',
+    ];
+    unsafe { std::str::from_utf8_unchecked(&SRC_LIB_RS) }
+};
+
 pub fn resources(res: impl AsRef<Path>) -> std::path::PathBuf {
     let path = Path::new("rstest").join(res.as_ref());
     super::resources(path)
@@ -21,8 +28,24 @@ fn run_test(res: impl AsRef<Path>) -> (std::process::Output, String) {
     )
 }
 
+fn prepare_folders_tests(project_path: std::path::PathBuf) {
+    let folders_path = project_path.join("folders");
+
+    let folder_with_txt_extension_path = folders_path.join("folder_but.txt");
+    let sub_folder_path = folders_path.join("sub");
+
+    std::fs::create_dir(&folders_path).unwrap();
+    std::fs::create_dir(&folder_with_txt_extension_path).unwrap();
+    std::fs::create_dir(&sub_folder_path).unwrap();
+
+    let text_file_path = sub_folder_path.join("file.txt");
+    File::create(text_file_path).unwrap();
+}
+
 #[test]
 fn files() {
+    std::env::set_var("FILES_ENV_VAR", "files");
+
     let prj = prj("files.rs");
     let files_path = prj.path().join("files");
     let sub_folder = files_path.join("sub");
@@ -31,6 +54,8 @@ fn files() {
     std::fs::create_dir(&sub_folder).unwrap();
     std::fs::create_dir(&up_sub_folder).unwrap();
 
+    prepare_folders_tests(prj.path());
+
     for n in 0..4 {
         let name = format!("element_{}.txt", n);
         let path = files_path.join(&name);
@@ -62,24 +87,69 @@ fn files() {
     let output = prj.run_tests().unwrap();
 
     TestResults::new()
-        .ok("start_with_name::path_1__UP_files_test_sub_folder_from_parent_folder_txt")
-        .ok("start_with_name::path_2_files_element_0_txt")
-        .ok("start_with_name::path_3_files_element_1_txt")
-        .ok("start_with_name::path_4_files_element_2_txt")
-        .ok("start_with_name::path_5_files_element_3_txt")
-        .ok("start_with_name::path_6_files_sub_sub_dir_file_txt")
-        .ok("start_with_name_with_include::path_1_files__ignore_me_txt")
-        .ok("start_with_name_with_include::path_2_files_element_0_txt")
-        .ok("start_with_name_with_include::path_3_files_element_1_txt")
-        .ok("start_with_name_with_include::path_4_files_element_2_txt")
-        .ok("start_with_name_with_include::path_5_files_element_3_txt")
-        .ok("start_with_name_with_include::path_6_files_sub_sub_dir_file_txt")
+        .ok("env_vars::path_1_files_element_0_txt")
+        .ok("env_vars::path_2_files_element_1_txt")
+        .ok("env_vars::path_3_files_element_2_txt")
+        .ok("env_vars::path_4_files_element_3_txt")
+        .ok("env_vars::path_5_files_sub_sub_dir_file_txt")
+        .ok("env_vars_base_dir::path_1_element_0_txt")
+        .ok("env_vars_base_dir::path_2_element_1_txt")
+        .ok("env_vars_base_dir::path_3_element_2_txt")
+        .ok("env_vars_base_dir::path_4_element_3_txt")
+        .ok("env_vars_base_dir::path_5_sub_sub_dir_file_txt")
+        .ok("env_vars_unknown::path_1_files_element_0_txt")
+        .ok("env_vars_unknown::path_2_files_element_1_txt")
+        .ok("env_vars_unknown::path_3_files_element_2_txt")
+        .ok("env_vars_unknown::path_4_files_element_3_txt")
+        .ok("env_vars_unknown::path_5_files_sub_sub_dir_file_txt")
+        .ok("ignore_missing_env_vars::path_1_files_element_0_txt")
+        .ok("ignore_missing_env_vars::path_2_files_element_1_txt")
+        .ok("ignore_missing_env_vars::path_3_files_element_2_txt")
+        .ok("ignore_missing_env_vars::path_4_files_element_3_txt")
+        .ok("ignore_missing_env_vars::path_5_files_sub_sub_dir_file_txt")
+        .ok("include_bytes::contents_1__UP_files_test_sub_folder_from_parent_folder_txt")
+        .ok("include_bytes::contents_2_files_element_0_txt")
+        .ok("include_bytes::contents_3_files_element_1_txt")
+        .ok("include_bytes::contents_4_files_element_2_txt")
+        .ok("include_bytes::contents_5_files_element_3_txt")
+        .ok("include_bytes::contents_6_files_sub_sub_dir_file_txt")
+        .ok("include_str::contents_1__UP_files_test_sub_folder_from_parent_folder_txt")
+        .ok("include_str::contents_2_files_element_0_txt")
+        .ok("include_str::contents_3_files_element_1_txt")
+        .ok("include_str::contents_4_files_element_2_txt")
+        .ok("include_str::contents_5_files_element_3_txt")
+        .ok("include_str::contents_6_files_sub_sub_dir_file_txt")
         .ok("module::pathbuf_need_not_be_in_scope::path_1_files__ignore_me_txt")
         .ok("module::pathbuf_need_not_be_in_scope::path_2_files_element_0_txt")
         .ok("module::pathbuf_need_not_be_in_scope::path_3_files_element_1_txt")
         .ok("module::pathbuf_need_not_be_in_scope::path_4_files_element_2_txt")
         .ok("module::pathbuf_need_not_be_in_scope::path_5_files_element_3_txt")
         .ok("module::pathbuf_need_not_be_in_scope::path_6_files_sub_sub_dir_file_txt")
+        .ok("start_with_name::path_1__UP_files_test_sub_folder_from_parent_folder_txt")
+        .ok("start_with_name::path_2_files_element_0_txt")
+        .ok("start_with_name::path_3_files_element_1_txt")
+        .ok("start_with_name::path_4_files_element_2_txt")
+        .ok("start_with_name::path_5_files_element_3_txt")
+        .ok("start_with_name::path_6_files_sub_sub_dir_file_txt")
+        .ok("start_with_name_file_mode::path_1__UP_files_test_sub_folder_from_parent_folder_txt")
+        .ok("start_with_name_file_mode::path_2_files_element_0_txt")
+        .ok("start_with_name_file_mode::path_3_files_element_1_txt")
+        .ok("start_with_name_file_mode::path_4_files_element_2_txt")
+        .ok("start_with_name_file_mode::path_5_files_element_3_txt")
+        .ok("start_with_name_file_mode::path_6_files_sub_sub_dir_file_txt")
+        .ok("start_with_name_with_include::path_1_files__ignore_me_txt")
+        .ok("start_with_name_with_include::path_2_files_element_0_txt")
+        .ok("start_with_name_with_include::path_3_files_element_1_txt")
+        .ok("start_with_name_with_include::path_4_files_element_2_txt")
+        .ok("start_with_name_with_include::path_5_files_element_3_txt")
+        .ok("start_with_name_with_include::path_6_files_sub_sub_dir_file_txt")
+        .ok("ignore_directories::path_1_folders_sub_file_txt")
+        .ok("ignore_directories_wildcard::path_1_folders_sub_file_txt")
+        .ok("include_directories::path_1_folders_folder_but_txt")
+        .ok("include_directories::path_2_folders_sub_file_txt")
+        .ok("include_directories_wildcard::path_1_folders_folder_but_txt")
+        .ok("include_directories_wildcard::path_2_folders_sub")
+        .ok("include_directories_wildcard::path_3_folders_sub_file_txt")
         .assert(output);
 }
 
@@ -165,8 +235,8 @@ fn impl_input() {
 }
 
 #[test]
-fn use_mutable_fixture_in_parametric_argumnts() {
-    let (output, _) = run_test("use_mutable_fixture_in_parametric_argumnts.rs");
+fn use_mutable_fixture_in_parametric_arguments() {
+    let (output, _) = run_test("use_mutable_fixture_in_parametric_arguments.rs");
 
     TestResults::new()
         .with_contains(true)
@@ -195,12 +265,11 @@ fn should_reject_no_item_function() {
         format!(
             "
         error: expected `fn`
-         --> {}/src/lib.rs:4:1
+         --> {name}{SRC_LIB_RS}:4:1
           |
         4 | struct Foo;
           | ^^^^^^
-        ",
-            name
+        "
         )
         .unindent()
     );
@@ -210,12 +279,11 @@ fn should_reject_no_item_function() {
         format!(
             "
         error: expected `fn`
-         --> {}/src/lib.rs:7:1
+         --> {name}{SRC_LIB_RS}:7:1
           |
         7 | impl Foo {{}}
           | ^^^^
-        ",
-            name
+        "
         )
         .unindent()
     );
@@ -225,12 +293,11 @@ fn should_reject_no_item_function() {
         format!(
             "
         error: expected `fn`
-          --> {}/src/lib.rs:10:1
+          --> {name}{SRC_LIB_RS}:10:1
            |
         10 | mod mod_baz {{}}
            | ^^^
-        ",
-            name
+        "
         )
         .unindent()
     );
@@ -295,21 +362,21 @@ mod dump_input_values {
 
         assert_all_in!(
             output.stderr.str(),
-            format!("--> {}/src/lib.rs:10:11", name),
+            format!("--> {name}{SRC_LIB_RS}:10:11"),
             "fn single(fixture: S) {}",
             "^^^^^^^ `S` cannot be formatted using `{:?}`"
         );
 
         assert_in!(
             output.stderr.str(),
-            format!("--> {}/src/lib.rs:15:10", name),
+            format!("--> {name}{SRC_LIB_RS}:15:10"),
             "fn cases(s: S) {}",
             "^ `S` cannot be formatted using `{:?}`"
         );
 
         assert_in!(
             output.stderr.str(),
-            format!("--> {}/src/lib.rs:20:11", name),
+            format!("--> {name}{SRC_LIB_RS}:20:11"),
             "fn matrix(s: S) {}",
             "^ `S` cannot be formatted using `{:?}`"
         );
@@ -431,6 +498,7 @@ mod single {
             .fail("sync_case_fail")
             .fail("sync_case_panic_fail")
             .ok("async_case")
+            .ok("async_case_test_attr")
             .ok("async_case_panic")
             .fail("async_case_fail")
             .fail("async_case_panic_fail")
@@ -579,6 +647,9 @@ mod cases {
             .ok("fn_async::case_1_pass")
             .ok("fn_async::case_2_panic")
             .fail("fn_async::case_3_fail")
+            .ok("fn_async_test_attr::case_1_pass")
+            .ok("fn_async_test_attr::case_2_panic")
+            .fail("fn_async_test_attr::case_3_fail")
             .assert(output);
     }
 
@@ -781,12 +852,11 @@ mod cases {
                 format!(
                     "
                 error: No cases for this argument.
-                 --> {}/src/lib.rs:3:10
+                 --> {name}{SRC_LIB_RS}:3:10
                   |
                 3 | #[rstest(one, two, three)]
                   |          ^^^
-                ",
-                    name
+                "
                 )
                 .unindent()
             );
@@ -904,6 +974,10 @@ mod matrix {
             .fail("fn_async::first_1")
             .fail("fn_async::first_2")
             .ok("fn_async::first_2")
+            .ok("fn_async_test_attr::first_1")
+            .fail("fn_async_test_attr::first_1")
+            .fail("fn_async_test_attr::first_2")
+            .ok("fn_async::first_2")
             .assert(output);
     }
 
@@ -973,6 +1047,27 @@ fn happy_path() {
         .assert(output);
 }
 
+#[test]
+fn destruct() {
+    let (output, _) = run_test("destruct.rs");
+
+    TestResults::new()
+        .ok("cases_destruct::case_1_two_times_twenty_one::__destruct_3_1_T__new_42_1_")
+        .ok("cases_destruct::case_1_two_times_twenty_one::__destruct_3_2_T_a_3_b_14_")
+        .ok("cases_destruct::case_2_six_times_seven::__destruct_3_1_T__new_42_1_")
+        .ok("cases_destruct::case_2_six_times_seven::__destruct_3_2_T_a_3_b_14_")
+        .ok("cases_destruct_named_tuple::case_1_two_times_twenty_one::__destruct_3_1_S_42_1_")
+        .ok("cases_destruct_named_tuple::case_1_two_times_twenty_one::__destruct_3_2_S_3_14_")
+        .ok("cases_destruct_named_tuple::case_2_six_times_seven::__destruct_3_1_S_42_1_")
+        .ok("cases_destruct_named_tuple::case_2_six_times_seven::__destruct_3_2_S_3_14_")
+        .ok("cases_destruct_tuple::case_1_two_times_twenty_one::__destruct_3_1__42_1_")
+        .ok("cases_destruct_tuple::case_1_two_times_twenty_one::__destruct_3_2__3_14_")
+        .ok("cases_destruct_tuple::case_2_six_times_seven::__destruct_3_1__42_1_")
+        .ok("cases_destruct_tuple::case_2_six_times_seven::__destruct_3_2__3_14_")
+        .ok("swapped")
+        .assert(output);
+}
+
 #[test]
 fn rename() {
     let (output, _) = run_test("rename.rs");
@@ -1004,6 +1099,25 @@ fn ignore_underscore_args() {
         .assert(output);
 }
 
+#[test]
+fn ignore_attributes_args_if_any() {
+    let prj = prj("ignore_attributes_args_if_any.rs");
+    prj.add_dependency(
+        "sqlx",
+        r#"{version="*", features=["sqlite","macros","runtime-tokio"]}"#,
+    );
+
+    let output = prj.run_tests().unwrap();
+
+    TestResults::new()
+        .with_contains(true)
+        .ok("test_sum::case_1::c_1_1")
+        .ok("test_sum::case_1::c_2_2")
+        .ok("test_sum::case_2::c_1_1")
+        .ok("test_sum::case_2::c_2_2")
+        .assert(output);
+}
+
 #[test]
 fn timeout() {
     let mut prj = prj("timeout.rs");
@@ -1072,8 +1186,8 @@ mod import_crate_with_other_name {
         prj.add_dependency(
             "other_name",
             &format!(
-                r#"{{path="{}", package = "rstest", default-features = {}, features = [{}]}}"#,
-                prj.exec_dir_str().as_str(),
+                r#"{{path={}, package = "rstest", default-features = {}, features = [{}]}}"#,
+                rstest_test::to_toml_string(prj.exec_dir_str()),
                 default_features,
                 features
             ),
@@ -1093,8 +1207,8 @@ mod import_crate_with_other_name {
         prj.add_dependency(
             "rstest",
             &format!(
-                r#"{{path="{}", default-features = false}}"#,
-                prj.exec_dir_str().as_str(),
+                r#"{{path={}, default-features = false}}"#,
+                rstest_test::to_toml_string(prj.exec_dir_str()),
             ),
         );
         let prj = prj.set_code_file(resources("convert_string_literal.rs"));
@@ -1131,7 +1245,7 @@ fn local_lifetime() {
 #[test]
 fn by_ref() {
     let prj = prj("by_ref.rs");
-    let files_path = prj.path().join("files");
+    let files_path: std::path::PathBuf = prj.path().join("files");
     std::fs::create_dir(&files_path).unwrap();
     let name = "my_name.txt";
     let mut out = File::create(files_path.join(name)).unwrap();
@@ -1145,6 +1259,51 @@ fn by_ref() {
         .assert(output);
 }
 
+#[test]
+fn no_std() {
+    let prj = prj("no_std.rs");
+    prj.add_dependency("async-std", r#"{version="*", features=["attributes"]}"#);
+    let output = prj.run_tests().unwrap();
+
+    TestResults::new()
+        .ok("it_works::case_1")
+        .ok("async_works")
+        .assert(output);
+}
+
+#[test]
+fn context() {
+    let (output, _) = run_test("context.rs");
+
+    TestResults::new()
+        .ok("with_case::case_1_description")
+        .ok("without_case")
+        .ok("first::inner::test")
+        .ok("measure_time")
+        .assert(output);
+}
+
+#[test]
+fn should_works_with_smol_async_runtime() {
+    let prj = prj("smol.rs");
+    prj.add_dependency("smol", r#""2.0.2""#);
+    prj.add_dependency("smol-macros", r#""0.1.1""#);
+    prj.add_dependency("macro_rules_attribute", r#""0.2.2""#);
+
+    let output = prj.run_tests().unwrap();
+
+    TestResults::new()
+        .with_contains(true)
+        .ok("async_test::case_1::c_1_2")
+        .ok("async_test::case_1::c_2_4")
+        .ok("async_test::case_1::c_3_16")
+        .ok("async_test::case_2::c_1_2")
+        .ok("async_test::case_2::c_2_4")
+        .ok("async_test::case_2::c_3_16")
+        .assert(output);
+}
+
+
 mod async_timeout_feature {
     use super::*;
 
@@ -1157,8 +1316,8 @@ mod async_timeout_feature {
         prj.add_dependency(
             "rstest",
             &format!(
-                r#"{{path="{}", default-features = false {}}}"#,
-                prj.exec_dir_str().as_str(),
+                r#"{{path={}, default-features = false {}}}"#,
+                rstest_test::to_toml_string(prj.exec_dir_str()),
                 features
             ),
         );
@@ -1214,10 +1373,9 @@ mod should_show_correct_errors {
             output.stderr.str(),
             format!(
                 "
-                  --> {}/src/lib.rs:13:33
+                  --> {name}{SRC_LIB_RS}:13:33
                    |
-                13 | fn error_cannot_resolve_fixture(no_fixture: u32, f: u32) {{}}",
-                name
+                13 | fn error_cannot_resolve_fixture(no_fixture: u32, f: u32) {{}}"
             )
             .unindent()
         );
@@ -1232,12 +1390,11 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Missed argument: 'not_a_fixture' should be a test function argument.
-                  --> {}/src/lib.rs:28:23
+                  --> {name}{SRC_LIB_RS}:28:23
                    |
                 28 | #[rstest(f, case(42), not_a_fixture(24))]
                    |                       ^^^^^^^^^^^^^
-                ",
-                name
+                "
             )
             .unindent()
         );
@@ -1252,11 +1409,10 @@ mod should_show_correct_errors {
             format!(
                 r#"
                 error[E0308]: mismatched types
-                 --> {}/src/lib.rs:9:18
+                 --> {name}{SRC_LIB_RS}:9:18
                   |
                 9 |     let a: u32 = "";
-                "#,
-                name
+                "#
             )
             .unindent()
         );
@@ -1271,11 +1427,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error[E0308]: mismatched types
-                  --> {}/src/lib.rs:16:29
+                  --> {name}{SRC_LIB_RS}:16:29
                    |
                 16 | fn error_fixture_wrong_type(fixture: String, f: u32) {{}}
-                ",
-                name
+                "
             )
             .unindent()
         );
@@ -1290,10 +1445,9 @@ mod should_show_correct_errors {
             format!(
                 "
                 error[E0308]: mismatched types
-                  --> {}/src/lib.rs:19:26
+                  --> {name}{SRC_LIB_RS}:19:26
                    |
-                19 | fn error_case_wrong_type(f: &str) {{}}",
-                name
+                19 | fn error_case_wrong_type(f: &str) {{}}"
             )
             .unindent()
         );
@@ -1308,10 +1462,9 @@ mod should_show_correct_errors {
             format!(
                 "
                 error[E0308]: mismatched types
-                  --> {}/src/lib.rs:51:28
+                  --> {name}{SRC_LIB_RS}:51:28
                    |
-                51 | fn error_matrix_wrong_type(f: &str) {{}}",
-                name
+                51 | fn error_matrix_wrong_type(f: &str) {{}}"
             )
             .unindent()
         );
@@ -1320,12 +1473,12 @@ mod should_show_correct_errors {
     #[test]
     fn if_arbitrary_rust_code_has_some_errors() {
         let (output, name) = execute();
+        let src_lib_rs = regex::escape(SRC_LIB_RS);
 
         assert_regex!(
             format!(
                 r#"error\[E0308\]: mismatched types
-                \s+--> {}/src/lib\.rs:22:31"#,
-                name
+                \s+--> {name}{src_lib_rs}:22:31"#
             )
             .unindent(),
             output.stderr.str()
@@ -1338,8 +1491,7 @@ mod should_show_correct_errors {
         assert_regex!(
             format!(
                 r#"error\[E0308\]: mismatched types
-                \s+--> {}/src/lib\.rs:53:45"#,
-                name
+                \s+--> {name}{src_lib_rs}:53:45"#
             )
             .unindent(),
             output.stderr.str()
@@ -1359,11 +1511,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'f' is already defined.
-                  --> {}/src/lib.rs:41:13
+                  --> {name}{SRC_LIB_RS}:41:13
                    |
                 41 | #[rstest(f, f(42), case(12))]
-                   |             ^",
-                name
+                   |             ^"
             )
             .unindent()
         );
@@ -1378,11 +1529,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'f' is already defined.
-                  --> {}/src/lib.rs:44:17
+                  --> {name}{SRC_LIB_RS}:44:17
                    |
                 44 | #[rstest(f(42), f, case(12))]
-                   |                 ^",
-                name
+                   |                 ^"
             )
             .unindent()
         );
@@ -1397,11 +1547,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'f' is already defined.
-                  --> {}/src/lib.rs:47:20
+                  --> {name}{SRC_LIB_RS}:47:20
                    |
                 47 | #[rstest(v, f(42), f(42), case(12))]
-                   |                    ^",
-                name
+                   |                    ^"
             )
             .unindent()
         );
@@ -1416,11 +1565,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Missed argument: 'not_exist_1' should be a test function argument.
-                  --> {}/src/lib.rs:61:10
+                  --> {name}{SRC_LIB_RS}:61:10
                    |
                 61 | #[rstest(not_exist_1 => [42],
-                   |          ^^^^^^^^^^^",
-                name
+                   |          ^^^^^^^^^^^"
             )
             .unindent()
         );
@@ -1430,11 +1578,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Missed argument: 'not_exist_2' should be a test function argument.
-                  --> {}/src/lib.rs:62:10
+                  --> {name}{SRC_LIB_RS}:62:10
                    |
                 62 |          not_exist_2 => [42])]
-                   |          ^^^^^^^^^^^",
-                name
+                   |          ^^^^^^^^^^^"
             )
             .unindent()
         );
@@ -1449,11 +1596,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'f' is already defined.
-                  --> {}/src/lib.rs:65:25
+                  --> {name}{SRC_LIB_RS}:65:25
                    |
                 65 | #[rstest(f => [41, 42], f(42))]
-                   |                         ^",
-                name
+                   |                         ^"
             )
             .unindent()
         );
@@ -1468,11 +1614,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'a' is already defined.
-                  --> {}/src/lib.rs:77:25
+                  --> {name}{SRC_LIB_RS}:77:25
                    |
                 77 | #[rstest(a => [42, 24], a => [24, 42])]
-                   |                         ^",
-                name
+                   |                         ^"
             )
             .unindent()
         );
@@ -1487,11 +1632,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'f' is already defined.
-                  --> {}/src/lib.rs:68:17
+                  --> {name}{SRC_LIB_RS}:68:17
                    |
                 68 | #[rstest(f(42), f => [41, 42])]
-                   |                 ^",
-                name
+                   |                 ^"
             )
             .unindent()
         );
@@ -1506,11 +1650,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'a' is already defined.
-                  --> {}/src/lib.rs:71:23
+                  --> {name}{SRC_LIB_RS}:71:23
                    |
                 71 | #[rstest(a, case(42), a => [42])]
-                   |                       ^",
-                name
+                   |                       ^"
             )
             .unindent()
         );
@@ -1525,11 +1668,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'a' is already defined.
-                  --> {}/src/lib.rs:74:21
+                  --> {name}{SRC_LIB_RS}:74:21
                    |
                 74 | #[rstest(a => [42], a, case(42))]
-                   |                     ^",
-                name
+                   |                     ^"
             )
             .unindent()
         );
@@ -1544,11 +1686,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Duplicate argument: 'a' is already defined.
-                  --> {}/src/lib.rs:80:13
+                  --> {name}{SRC_LIB_RS}:80:13
                    |
                 80 | #[rstest(a, a, case(42))]
-                   |             ^",
-                name
+                   |             ^"
             )
             .unindent()
         );
@@ -1563,11 +1704,10 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Values list should not be empty
-                  --> {}/src/lib.rs:58:19
+                  --> {name}{SRC_LIB_RS}:58:19
                    |
                 58 | #[rstest(empty => [])]
-                   |                   ^^",
-                name
+                   |                   ^^"
             )
             .unindent()
         );
@@ -1577,7 +1717,7 @@ mod should_show_correct_errors {
     fn if_try_to_convert_literal_string_to_a_type_that_not_implement_from_str() {
         let (output, name) = execute();
 
-        assert_in!(output.stderr.str(), format!("--> {}/src/lib.rs:84:1", name));
+        assert_in!(output.stderr.str(), format!("--> {name}{SRC_LIB_RS}:84:1"));
         assert_in!(
             output.stderr.str(),
             "| -------- doesn't satisfy `S: FromStr`"
@@ -1592,12 +1732,11 @@ mod should_show_correct_errors {
             output.stderr.str(),
             format!(
                 "
-                  --> {}/src/lib.rs:93:8
+                  --> {name}{SRC_LIB_RS}:93:8
                    |
                 93 |     s: impl AsRef<str>,
                    |        ^^^^^^^^^^^^^^^
-                ",
-                name
+                "
             )
             .unindent()
         );
@@ -1611,12 +1750,11 @@ mod should_show_correct_errors {
             output.stderr.str(),
             format!(
                 "
-                   --> {}/src/lib.rs:102:5
+                   --> {name}{SRC_LIB_RS}:102:5
                     |
                 102 |     #[future]
                     |     ^^^^^^^^^
-                ",
-                name
+                "
             )
             .unindent()
         );
@@ -1631,12 +1769,11 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: expected attribute arguments in parentheses: #[timeout(...)]
-                   --> {}/src/lib.rs:108:3
+                   --> {name}{SRC_LIB_RS}:108:3
                     |
                 108 | #[timeout]
                     |   ^^^^^^^
-                ",
-                name
+                "
             )
             .unindent()
         );
@@ -1651,12 +1788,11 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: expected an expression
-                   --> {}/src/lib.rs:112:17
+                   --> {name}{SRC_LIB_RS}:112:17
                     |
                 112 | #[timeout(some -> strange -> invalid -> expression)]
                     |                 ^
-                ",
-                name
+                "
             )
             .unindent()
         );
@@ -1671,8 +1807,7 @@ mod should_show_correct_errors {
             format!(
                 "
                 error[E0308]: mismatched types
-                   --> {}/src/lib.rs:116:11",
-                name
+                   --> {name}{SRC_LIB_RS}:116:11"
             )
             .unindent()
         );
@@ -1696,8 +1831,7 @@ mod should_show_correct_errors {
             format!(
                 "
                 error: Invalid glob path: path contains non-relative component
-                   --> {}/src/lib.rs:120:30",
-                name
+                   --> {name}{SRC_LIB_RS}:120:30"
             )
             .unindent()
         );
@@ -1711,4 +1845,38 @@ mod should_show_correct_errors {
             .unindent()
         );
     }
+
+    #[test]
+    fn try_to_destruct_implicit_fixture() {
+        let (output, name) = execute();
+
+        assert_in!(
+            output.stderr.str(),
+            format!(
+                r#"
+                error: To destruct a fixture you should provide a path to resolve it by '#[from(...)]' attribute.
+                   --> {name}{SRC_LIB_RS}:126:27
+                    |
+                126 | fn wrong_destruct_fixture(T(a, b): T, #[with(42)] T(c, d): T) {{}}
+                    |                           ^^^^^^^^^^"#,
+            )
+            .unindent()
+        );
+
+        assert_in!(
+            output.stderr.str(),
+            format!(
+                r#"
+                error: To destruct a fixture you should provide a path to resolve it by '#[from(...)]' attribute.
+                   --> {name}{SRC_LIB_RS}:126:51
+                    |
+                126 | fn wrong_destruct_fixture(T(a, b): T, #[with(42)] T(c, d): T) {{}}
+                    |                                                   ^^^^^^^^^^"#,
+            )
+            .unindent()
+        );
+
+        assert_not_in!(output.stderr.str(), "#[case] T(e, f): T");
+        assert_not_in!(output.stderr.str(), "#[values(T(1, 2))] T(g, h): T");
+    }
 }
